package com.example.loadfile;

import lombok.RequiredArgsConstructor;
import org.springframework.http.*;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import static org.springframework.http.MediaType.MULTIPART_FORM_DATA_VALUE;

@RestController
@RequestMapping("users")
@RequiredArgsConstructor
@CrossOrigin
public class UserLibraryController {
    private final UserDao userDao;
    private final ImageDao imageDao;
    @GetMapping("helloWord")
    public String helloWord(@RequestParam String name) {
        return "Coucou " + name + " ! Tu vas bien ?";
    }

    @GetMapping("")
    public List<User> getUsers() {
        Iterable<User> it = userDao.findAll();
        List <User> users = new ArrayList<>();
        it.forEach(users::add);
        return users;
    }

    @PostMapping("")
    public List<User> addUser(@RequestBody User user) {
        userDao.save(user);
        return getUsers();
    }

    @PostMapping(value ="/{id}/file", consumes = MULTIPART_FORM_DATA_VALUE)
    @Transactional
    public User addFileToUser(@ModelAttribute AddFileToUserRequest addFileToUserRequest, @PathVariable Integer id) {
        User user = userDao.findById(id).orElseThrow(() -> new NoSuchElementException("No user with this id : " + id));
        Image image;
        try {
            image = Image.builder().bytes(addFileToUserRequest.file().getBytes()).build();
        } catch (IOException e) {
            throw new RuntimeException("An error occurred with the file");
        }
        imageDao.save(image);
        user.setImage(image);
        return user;
    }

    @GetMapping(value = "{id}/image", produces = MediaType.ALL_VALUE)
    public ResponseEntity<byte[]> getImage(@PathVariable Integer id) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentDisposition(ContentDisposition.builder("attachement").filename("filename.png").build());
        headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
        return this.userDao.findById(id)
                .map(User::getImage)
                .map(image -> new ResponseEntity<>(image.getBytes(), headers, HttpStatus.OK))
                .orElse(null);
    }

}

